<?php

namespace Drupal\cosign\Authentication\Provider;

use Drupal\Core\Authentication\AuthenticationProviderInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\user\UserAuthInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\cosign\CosignFunctions\CosignSharedFunctions;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Event\GetResponseForExceptionEvent;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Url;

/**
 * Cosign authentication provider.
 */
class Cosign implements AuthenticationProviderInterface {
  use StringTranslationTrait;
  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The user auth service.
   *
   * @var \Drupal\user\UserAuthInterface
   */
  protected $userAuth;

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityManagerInterface
   */
  protected $entityManager;
  /**
   * Provides the Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a Cosign provider object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\user\UserAuthInterface $user_auth
   *   The user authentication service.
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   Brings Translation into the class.
   */
  public function __construct(ConfigFactoryInterface $config_factory, UserAuthInterface $user_auth, EntityManagerInterface $entity_manager, MessengerInterface $messenger, TranslationInterface $string_translation) {
    $this->configFactory = $config_factory;
    $this->userAuth = $user_auth;
    $this->entityManager = $entity_manager;
    $this->messenger = $messenger;
    $this->stringTranslation = $string_translation;
  }

  /**
   * {@inheritdoc}
   */
  public function applies(Request $request) {
    $username = CosignSharedFunctions::cosignUserMasqueradeCheck();
    $drupal_user = user_load_by_name($username);
    // This session variable is set and sticks even after user_logout() causing
    // numerous problems. if we put cosign module priority after the user module
    // (priority 0 or below in services.yml) the symfony session sticks and the
    // previous user gets logged in. if we put it above the user module (above
    // priority 0) the user gets relogged
    // in every time because drupal's session hasn't been set yet...even though
    // symfony's has.
    // TODO This should be the proper way to get this but it doesnt get it -
    // $symfony_uid = $request->getSession()-> get('_sf2_attributes');.
    if (isset($_SESSION['_sf2_attributes']['uid']) && !empty($drupal_user) && $drupal_user->id() == $_SESSION['_sf2_attributes']['uid']) {
      // The user is already logged in. symfony knows, drupal doesnt yet.
      // bypass cosign so we dont login again.
      return FALSE;
    }
    if (CosignSharedFunctions::cosignIsHttps() &&
        $request->getRequestUri() != '/user/logout' &&
        ($this->configFactory->get('cosign.settings')->get('cosign_allow_cosign_anons') == 0 ||
        $this->configFactory->get('cosign.settings')->get('cosign_allow_anons_on_https') == 0 ||
        strpos($request->headers->get('referer'), $this->configFactory->get('cosign.settings')->get('cosign_login_path')) !== FALSE ||
        strpos($request->getRequestUri(), 'user/login') ||
        strpos($request->getRequestUri(), 'user/register'))
       ) {
      return TRUE;
    }
    else {
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function authenticate(Request $request) {
    $username = CosignSharedFunctions::cosignUserMasqueradeCheck();
    if ($user = CosignSharedFunctions::cosignUserStatus($username)) {
      return $user;
    }
    else {
      if (!CosignSharedFunctions::cosignIsFriendAccount($username)) {
        $login_url = Url::fromRoute('user.login')->toString();
        $cosign_brand = $this->configFactory->get('cosign.settings')->get('cosign_branded');
        $msg = $this->t('This site is restricted. You may try <a href="%login_url">logging in to %brand</a>.', ['login_url' => $login_url, '%brand' => $cosign_brand]);
        $this->messenger->get('messenger')->addMessage($msg, 'error');
      }
      throw new AccessDeniedHttpException();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function cleanup(Request $request) {}

  /**
   * {@inheritdoc}
   */
  public function handleException(GetResponseForExceptionEvent $event) {
    $exception = $event->getException();
    if ($exception instanceof AccessDeniedHttpException) {
      return TRUE;
    }
    return FALSE;
  }

}
