<?php

namespace Drupal\cosign\Controller;

use Symfony\Cmf\Component\Routing\RouteObjectInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\cosign\CosignFunctions\CosignSharedFunctions;
use Symfony\Component\HttpFoundation\Cookie;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\FormBuilder;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Entity\EntityManagerInterface;

/**
 * The Cosign Controller.
 */
class CosignController extends ControllerBase {
  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;
  /**
   * Drupal\Core\Session\AccountProxy definition.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;
  /**
   * Drupal\Core\Form\FormInterface definition.
   *
   * @var Drupal\Core\Form\FormInterface
   */
  protected $formBuilder;
  /**
   * Provides the Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;
  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityManagerInterface
   */
  protected $entityManager;

  /**
   * Constructs a new Cosign Controller object.
   */
  public function __construct(ConfigFactoryInterface $configFactory, AccountProxyInterface $currentUser, FormBuilder $formBuilder, EntityManagerInterface $entityManager, MessengerInterface $messenger) {
    $this->configFactory = $configFactory;
    $this->currentUser = $currentUser;
    $this->formBuilder = $formBuilder;
    $this->entityManager = $entityManager;
    $this->messenger = $messenger;
  }

  /**
   * Creates the static objects.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The Container interface.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('form_builder'),
      $container->get('entity.manager'),
      $container->get('messenger')
    );
  }

  /**
   * The Cosign Logout function.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The Request interface.
   */
  public function cosignLogout(Request $request) {
    $uname = $this->currentUser->getAccountName();
    if ($this->configFactory->get('cosign.settings')->get('cosign_allow_cosign_anons') == 0 ||
       (CosignSharedFunctions::cosignIsFriendAccount($uname) &&
       $this->configFactory->get('cosign.settings')->get('cosign_allow_friend_accounts') == 0)
      ) {
      // Logs you out.
      $response = CosignController::cosignCosignLogout();
    }
    else {
      $cosign_brand = $this->configFactory->get('cosign.settings')->get('cosign_branded');
      $request->attributes->get(RouteObjectInterface::ROUTE_OBJECT)->setDefault('_title', 'Would you like to logout of ' . $cosign_brand . ' also?');
      $form_class = '\Drupal\cosign\Form\CosignLogout';
      $response['form'] = $this->formBuilder->getForm($form_class);
    }
    if (!empty($uname)) {
      $this->messenger->addMessage('User ' . $uname . ' successfully logged out of ' . $this->configFactory->get("system.site")->get("name"), 'status', FALSE);
    }
    return $response;
  }

  /**
   * Send this over to an event handler after forcing https.
   */
  public function cosignLogin(Request $request) {
    $request_uri = $request->getRequestUri();
    global $base_path;
    if (!CosignSharedFunctions::cosignIsHttps()) {
      return new TrustedRedirectResponse('https://' . $_SERVER['HTTP_HOST'] . $request_uri);
    }
    else {
      if ($request_uri == $base_path) {
        // The front page is set to /user. we have to login here to avoid a
        // redirect loop.
        $username = CosignSharedFunctions::cosignUserMasqueradeCheck();
        $user = CosignSharedFunctions::cosignUserStatus($username);
        if (empty($user) || $user->id() == 0) {
          $cosign_brand = $this->configFactory->get('cosign.settings')->get('cosign_branded');
          $markup = $this->t('<p>This site does not auto create users from %cosignBrand. Please contact the <a href="mailto:%email">site administrator</a> to have an account created.</p>', ['%cosignBrand' => $cosign_brand, '%email' => $this->configFactory->get("system.site")->get("mail")]);
          $response = [
            '#type' => 'markup',
            '#title' => 'Auto creation of user accounts is disabled.',
            '#markup' => $markup,
          ];
          return $response;
        }
        else {
          // Admin role can now be named anything.
          $is_admin = array_intersect($this->entityManager->entityQuery('user_role')->condition('is_admin', TRUE)->execute(), $user->getRoles());
          if (!empty($is_admin) && $this->configFactory->get('cosign.settings')->get('cosign_allow_anons_on_https') == 1) {
            $this->messenger->addMessage('When the homepage is set to /user (Drupal default), anonymous browsing will not always work', 'warning');
          }
          $referrer = $base_path . 'user';
        }
      }
      elseif (isset($_SERVER['HTTP_REFERER'])) {
        $referrer = $_SERVER['HTTP_REFERER'];
      }
      else {
        $referrer = $base_path;
      }

      return new TrustedRedirectResponse($referrer);
    }
  }

  /**
   * Logs someone out.
   *
   * @return \Drupal\Core\Routing\TrustedRedirectResponse
   *   The response cookie.
   */
  public function cosignCosignLogout() {
    $logout = CosignSharedFunctions::cosignLogoutUrl();
    if ($this->currentUser->isAuthenticated()) {
      user_logout();
    }
    $response = new TrustedRedirectResponse($logout);
    // This had to be done of user was logged into cosign/drupal for several
    // minutes after logging out
    // for ref - Cookie($name, $value, $minutes, $path, $domain, $secure,
    // $httpOnly)
    // set value to nonsense and domain to blank so it becomes a host cookie.
    $response->headers->setCookie(new Cookie('cosign-' . $_SERVER['HTTP_HOST'], 'jibberish', 0, '/', '', -1, 0));
    return $response;
  }

}
