<?php

namespace Drupal\cosign\CosignFunctions;

use Drupal\user\Entity\User;

/**
 * Cosign shared functions.
 */
class CosignSharedFunctions {

  /**
   * Check whether user is logged in.
   *
   * Determines is user is logged into cosign, is a drupal user, and is logged
   * into drupal.
   *
   * @return \Drupal\user\Entity\User
   *   User object.
   */
  public static function cosignUserStatus($cosign_username) {
    $user = \Drupal::currentUser();
    $uname = $user->getAccountName();
    $drupal_user = user_load_by_name($cosign_username);
    if ($drupal_user && $drupal_user->isBlocked()) {
      if (\Drupal::config('cosign.settings')->get('cosign_allow_anons_on_https') == 1) {
        return User::load(0);
      }
      else {
        return NULL;
      }
    }
    if (!empty($uname)) {
      // Youre already logged in
      // make sure you are the cosign user. if not log out. This is unlikely.
      if ($cosign_username != $uname) {
        user_logout();
        return NULL;
      }
    }
    if (!empty($cosign_username)) {
      $is_friend_account = CosignSharedFunctions::cosignIsFriendAccount($cosign_username);
      // If friend accounts are not allowed, log them out.
      if (\Drupal::config('cosign.settings')->get('cosign_allow_friend_accounts') == 0 && $is_friend_account) {
        CosignSharedFunctions::cosignFriendNotAllowed();
        if (\Drupal::config('cosign.settings')->get('cosign_allow_anons_on_https') == 1) {
          return User::load(0);
        }
        else {
          return NULL;
        }
      }
    }
    if (!empty($cosign_username) && !empty($drupal_user) && empty($uname)) {
      // Login the cosign user.
      CosignSharedFunctions::cosignLoginUser($drupal_user);
    }
    elseif (!empty($cosign_username) && empty($drupal_user)) {
      // Cosign user doesn't have a drupal account.
      if (\Drupal::config('cosign.settings')->get('cosign_autocreate') == 1) {
        $new_user = CosignSharedFunctions::cosignCreateNewUser($cosign_username);
        User::load($new_user->id());
      }
      else {
        User::load(0);
      }
    }
    elseif (empty($cosign_username) && \Drupal::config('cosign.settings')->get('cosign_allow_anons_on_https') == 0) {
      // No cosign account found.
      user_logout();
      return NULL;
    }
    $user = \Drupal::currentUser();
    if (!$user) {
      $user = User::load(0);
    }
    if ($user->id() == 0 && \Drupal::config('cosign.settings')->get('cosign_allow_anons_on_https') == 1) {
    }
    return $user;
  }

  /**
   * Logs cosign user into drupal.
   *
   * @return \Drupal\user\Entity\User
   *   User object.
   */
  public static function cosignLoginUser($drupal_user) {
    user_login_finalize($drupal_user);
    $the_user = \Drupal::currentUser();
    $username = CosignSharedFunctions::cosignUserMasqueradeCheck();

    if ($the_user->getAccountName() != $username) {
      \Drupal::logger('cosign')->notice('User attempted login and the cosign username: @remote_user, did not match the drupal username: @drupal_user', ['@remote_user' => $username, '@drupal_user' => $the_user->getAccountName()]);
      user_logout();
    }

    return User::load($the_user->id());
  }

  /**
   * Performs tasks if friend accounts aren't allowed.
   */
  public static function cosignFriendNotAllowed() {
    \Drupal::logger('cosign')->notice('User attempted login using a university friend account and the friend account configuration setting is turned off: @remote_user', ['@remote_user' => '']);
    \Drupal::messenger()->addMessage(t('%message', ['%message' => \Drupal::config('cosign.settings')->get('cosign_friend_account_message')]), 'warning');
    if (\Drupal::config('cosign.settings')->get('cosign_allow_anons_on_https') == 1) {
      $cosign_brand = \Drupal::config('cosign.settings')->get('cosign_branded');
      \Drupal::messenger()->addMessage(t('You might want to <a href="/user/logout">logout of %cosignBrand</a> to browse anonymously or as another %cosignBrand user.', ['%cosignBrand' => $cosign_brand]), 'warning');
    }
    else {
      user_logout();
    }
  }

  /**
   * Returns a logout URL.
   */
  public static function cosignLogoutUrl() {
    $logout_path = \Drupal::config('cosign.settings')->get('cosign_logout_path');
    $logout_to = \Drupal::config('cosign.settings')->get('cosign_logout_to') . '/';
    return $logout_path . '?' . $logout_to;
  }

  /**
   * Attempts to retrieve the remote user from the $_SERVER variable.
   *
   * If the user is logged in to cosign webserver auth, the remote user variable
   * will contain the name of the user logged in.
   *
   * @return string
   *   String username or empty string.
   */
  public static function cosignRetrieveRemoteUser() {
    $cosign_name = '';
    // Make sure we get the remote user whichever way it is available.
    if (isset($_SERVER['REDIRECT_REMOTE_USER'])) {
      $cosign_name = $_SERVER['REDIRECT_REMOTE_USER'];
    }
    elseif (isset($_SERVER['REMOTE_USER'])) {
      $cosign_name = $_SERVER['REMOTE_USER'];
    }

    return $cosign_name;
  }

  /**
   * Attempts to retrieve the protossl from the $_SERVER variable.
   *
   * We need to check for https on logins.
   * since we need to intercept redirects from routes and events, this is a
   * shared function.
   *
   * @return bool
   *   Boolean TRUE or FALSE.
   */
  public static function cosignIsHttps() {
    $is_https = FALSE;
    if (\Drupal::request()->server->get('protossl') == 's') {
      $is_https = TRUE;
    }

    return $is_https;
  }

  /**
   * Attempts to retrieve the remote realm from the $_SERVER variable.
   *
   * If the user is logged in to cosign webserver auth, the remote realm
   * variable will contain friend or UMICH.EDU (or some other implemetation).
   *
   * @return bool
   *   Boolean TRUE or FALSE.
   */
  public static function cosignIsFriendAccount($username) {
    // Make sure we get friend whichever way it is available.
    $is_friend_account = FALSE;
    if ((isset($_SERVER['REMOTE_REALM']) && $_SERVER['REMOTE_REALM'] == 'friend') || stristr($username, '@')) {
      $is_friend_account = TRUE;
    }

    return $is_friend_account;
  }

  /**
   * Creates a new drupal user.
   *
   * Creates a new drupal user with the cosign username and email address with
   * domain from admin form.
   *
   * @return \Drupal\user\Entity\User
   *   User object.
   */
  public static function cosignCreateNewUser($cosign_name) {
    if (\Drupal::config('cosign.settings')->get('cosign_autocreate') == 1) {
      $new_user = [];
      $new_user['name'] = $cosign_name;
      $new_user['status'] = 1;
      $new_user['password'] = user_password();
      if (CosignSharedFunctions::cosignIsFriendAccount($cosign_name)) {
        // Friend account.
        $new_user['mail'] = $cosign_name;
      }
      else {
        $new_user['mail'] = $cosign_name . '@' . \Drupal::config('cosign.settings')->get('cosignautocreate_email_domain');
      }
      $account = User::create($new_user);
      $account->enforceIsNew();
      $account->save();

      return CosignSharedFunctions::cosignLoginUser($account);
    }
  }

  /**
   * Used to enable the Masquerade module to play nice w/ cosign.
   *
   * Whenever we need to get the username, we check to see if we're
   * masquerading. If we are, we use the masquerade session info to select our
   * user. Otherwise, we pull the remote cosign user.
   *
   * @return string
   *   The username.
   */
  public static function cosignUserMasqueradeCheck() {
    if (isset($_SESSION['masquerading']) && isset($_SESSION['_sf2_attributes']['uid'])) {
      $account = User::load(\Drupal::service('session')->get('uid'));
      $username = $account->getAccountName();
    }
    else {
      $username = CosignSharedFunctions::cosignRetrieveRemoteUser();
    }
    return $username;
  }

}
